#include "colorspace.hh"
#include <cmath>

using std::fmod;

class HSV : public Colorspace
{
    double Hue, Saturation, Value;
public:
    virtual void Set(const RGB &rgb)
    {
        double Max = rgb.r; if(rgb.g>Max)Max=rgb.g; if(rgb.b>Max)Max=rgb.b;
        double Min = rgb.r; if(rgb.g<Min)Min=rgb.g; if(rgb.b<Min)Min=rgb.b;
        double Delta = Max-Min;
        Value = Max;
        if(Max != 0.0)
            Saturation = Delta / Max;
        else
            Saturation = 0;
        if(Saturation == 0.0)
            Hue = 0.0;
        else
        {
            /**/ if(rgb.r == Max) Hue = (rgb.g - rgb.b) / Delta;
            else if(rgb.g == Max) Hue=2+(rgb.b - rgb.r) / Delta;
            else if(rgb.b == Max) Hue=4+(rgb.r - rgb.b) / Delta;
            Hue /= 6.0;
            if(Hue < 0)++Hue;
        }
    }
    virtual RGB Get() const
    {
        RGB tmp;
        if(Saturation == 0.0)
            tmp.r = tmp.g = tmp.b = 0;
        else
        {
            double h6 = Hue*6;
            double h = fmod(h6, 1);
            int i = (int)h;
            double f = h - i;
            double x = Value * (1 - Saturation);
            double y = Value * (1 - (Saturation * f));
            double z = Value * (1 - (Saturation * (1 - f)));
            switch(i)
            {
                case 0: tmp.r = Value; tmp.g = z; tmp.b = x; break;
                case 1: tmp.r = y; tmp.g = Value; tmp.b = x; break;
                case 2: tmp.r = x; tmp.g = Value; tmp.b = z; break;
                case 3: tmp.r = x; tmp.g = y; tmp.b = Value; break;
                case 4: tmp.r = z; tmp.g = x; tmp.b = Value; break;
                case 5: tmp.r = Value; tmp.g = x; tmp.b = y; break;
            }
        }
        return tmp;
    }
    virtual const char *Name1() const { return "Hue"; }
    virtual const char *Name2() const { return "Saturation"; }
    virtual const char *Name3() const { return "Value"; }
    virtual double Value1() const { return Hue; }
    virtual double Value2() const { return Saturation; }
    virtual double Value3() const { return Value; }
    virtual void SetValue1(double v) { Hue=v; }
    virtual void SetValue2(double v) { Saturation=v; }
    virtual void SetValue3(double v) { Value=v; }
};
